module Messages where

import Terms
import Rules
import Trie
import TPTP
import Horn

unknownMessage str = "moca: unknown option '" ++ str ++ "'.\n"

helpMessage = "Moca v0.2\n"++ 
            "Usage: ./moca <options> <file>\n\n" ++ 
              "\t--siav:\t\tuse the split-if encoding using all variables in s = t => u = v  \n" ++
              "\t--smsi:\t\tskip execution for parameters with the multi-split-if encoding mode\n" ++
              "\t-nap:\t\texecute without approximations by projection\n" ++
              "\t-nag:\t\texecute without approximations by generalization\n" ++
              "\t-help:\t\tDisplay this list of options\n" ++
              "\t--help:\t\tDisplay this list of options\n" 

errorMessage_SpecifyFilename = "Please specify filename. For Instance \n  $ ./moca problem.trs"

errorMessage_NoZ3 = "Z3 cannot be found."

resultForINF_Infeasible :: [HornClause Atom] -> ES -> ES -> OTRS -> String -> String
resultForINF_Infeasible hc es gs otrs remark = "YES\n\n" ++ 
  "proof:\n" ++ 
  "The input problem is infeasible because\n\n" ++
  "[1] the following set of Horn clauses is satisfiable:\n\n"++
  showHornClause hc ++ "\n\n" ++
  "This holds because\n\n" ++
  "[2] the following E does not entail the following G (Claessen-Smallbone's transformation (2018)):\n\n" ++
  "E:\n" ++ showES es ++ "\nG:\n" ++ showES gs ++ "\n\n" ++
  "This holds because\n\n" ++
  "[3] the following ground-complete ordered TRS entails E but does not entail G:\n" ++
  showOTRS otrs ++ "\n" ++
  remark

resultForINF_Maybe :: OTRS -> String -> String
resultForINF_Maybe otrs remark = "MAYBE\n" ++ remark

resultForINF_EmptyInput :: String 
resultForINF_EmptyInput = "ERROR\nNo condition."



resultForTPTP_SAT :: [HornClause Atom] -> ES -> ES -> OTRS -> String -> String
resultForTPTP_SAT hc es gs otrs remark =
  "% SZS status Satisfiable\n" ++
  "% SZS output start Proof\n" ++
  "The input problem is satisfiable because\n\n" ++
  "[1] the following set of Horn clauses is satisfiable:\n\n"++
  showHornClause hc ++ "\n\n" ++
  "This holds because\n\n" ++
  "[2] the following E does not entail the following G (Claessen-Smallbone's transformation (2018)):\n\n" ++
  "E:\n" ++ showES es ++ "\nG:\n" ++ showES gs ++ "\n\n" ++
  "This holds because\n\n" ++
  "[3] the following ground-complete ordered TRS entails E but does not entail G:\n\n" ++
  showOTRS otrs ++ "\n" ++
  remark ++ "\n" ++
  "% SZS output end Proof\n"

resultForTPTP_UNSAT :: [HornClause Atom] -> ES -> ES -> OTRS -> String -> String
resultForTPTP_UNSAT hc es gs otrs remark =
  "% SZS status Unsatisfiable\n" ++
  "% SZS output start Proof\n" ++
  "The input problem is unsatisfiable because\n\n" ++
  "[1] the following set of Horn clauses is unsatisfiable:\n\n"++
  showHornClause hc ++ "\n\n" ++
  "This holds because\n\n" ++
  "[2] the following E entails the following G (Claessen-Smallbone's transformation (2018)):\n\n" ++
  "E:\n" ++ showES es ++ "\nG:\n" ++ showES gs ++ "\n\n" ++
  "This holds because\n\n" ++
  "[3] E entails the following ordered TRS and the lhs and rhs of G join by the TRS:\n\n" ++
  showOTRS otrs ++ "\n" ++
  remark ++ "\n" ++
  "% SZS output end Proof\n"
  
resultForTPTP_EmptyInput :: String 
resultForTPTP_EmptyInput = 
  "% SZS status Satisfiable\n" ++ 
  "% SZS output start Proof\n" ++
  "There is no formula in the input." ++
  "% SZS output end Proof\n"



  